"""
deleteVars.py
deleteVars(namelist=None, pattern=None) deletes variables included in namelist
or matching the pattern (a Regular Expression).

Example of use:
# You should first save a copy of deleteVars.py to your
# "C:\Python24\Lib\site-packages\" folder

GET FILE='c:\Program files\SPSS\employee data.sav'.
NUMERIC var1 TO var10.
BEGIN PROGRAM.
from deleteVars import *
deleteVars(namelist='gender var4 salary', pattern=r'^var[0-9]+')
END PROGRAM.
"""

import spss, spssaux
__author__  =  'Raynald Levesque raynald@spsstools.net'
__version__ =  '1.0'

#History
# v 1.0 done 2007/03/10

def deleteVars(namelist=None, pattern=None):
    """	usage:
    deleteVars(namelist='var1 var2 var3') deletes variables var1 var2 var3.
    deleteVars(pattern='.*sal') pattern is a regular expression, this example
    deletes variables containing 'sal'.
    Nothing happens (no errors) for variables that do not exist.
    Pending transformations are EXECUTEd before attempting to delete variables.
    """
    if namelist==None and pattern==None:
        print '*** No arguments were supplied'
        return

    cmd = ['EXECUTE.']
    vlist1 = []
    vlist2 = []

    if namelist != None:
        varDict = spssaux.VariableDict(namelist=namelist)
        vlist1 = [var for var in varDict.variables]
    if pattern != None:
        varDict = spssaux.VariableDict(pattern=pattern)
        vlist2 = [var for var in varDict.variables]
    vlist1.extend(vlist2)
    vset = set(vlist1)

    if not vset:
        print '*** none of the target variables existed'
    for v in vset:
        cmd.extend(['DELETE VAR %s.' % v])
    if cmd != ['EXECUTE.']:
        cmd = '\n'.join(cmd)
        try:
            spss.Submit(cmd)
        except:
            print '\n*** Following syntax generated an error',cmd
            raise
    return

if __name__ == '__main__':
    spss.Submit(r"""
        SET PRINTBACK=YES /MPRINT=YES.
        GET FILE='c:\Program files\SPSS\employee data.sav'.
        NUMERIC var1 TO var10. """)

    deleteVars(namelist='gender var4 salary', pattern=r'^var[0-9]+')
    spss.Submit('LIST /CASES=FROM 1 TO 5.')
