#!/usr/bin/env python
"""graphSize.py
Graph size depends on number of panels
May 7 2007"""

__author__  =  'Raynald Levesque raynald@spsstools.net'

import spss, spssaux, random

def doGraphSize(title, xvar, yvar, rowvar=None, function='mean',begIncrease = 4,
        endIncrease = 8, maxFactY = 0.5):
    """ Vary the length of a GGRAPH depending on the number of panels
    rowvar = the name of the variable defining row panels
    begIncrease = length increases if there are more than begIncrease panels
    endIncrease = length is at its maximum when there are > endIncrease panels
    maxFactY = 0.5  max increase length is by up to maxFactY (0.5 means 50% increase)
    """
    nbPanels = findNbDistinctVal(rowvar)

    xlen = 625
    ylen = 500
    function = function.lower()
    if begIncrease < nbPanels < endIncrease:
        ylen = ylen*(1.0 + maxFactY*(nbPanels - begIncrease)/endIncrease)
    elif nbPanels > begIncrease:
        ylen = ylen * (1.0 + maxFactY)

    title = title.replace('"', r'\"')
    #PAGE: begin(scale(750px,600px)) size=120%
    #PAGE: begin(scale(625px,500px)) size=100%

    cmd = r"""
        GGRAPH
          /GRAPHDATASET NAME="graphdataset" VARIABLES=%(xvar)s %(yvar)s %(rowvar)s
           MISSING=LISTWISE REPORTMISSING=NO
          /GRAPHSPEC SOURCE=INLINE LABEL="%(title)s".
        BEGIN GPL
        PAGE: begin(scale(%(xlen)spx,%(ylen)spx))
        SOURCE: s=userSource(id("graphdataset"))
        DATA: %(xvar)s=col(source(s), name("%(xvar)s"), unit.category())
        DATA: %(yvar)s=col(source(s), name("%(yvar)s"))
        DATA: %(rowvar)s=col(source(s), name("%(rowvar)s"), unit.category())
        COMMENT: dim(1)=xvar, dim(2)=yvar, dim(4)=rowvar
        GUIDE: axis(dim(1))
        GUIDE: axis(dim(2))
        GUIDE: axis(dim(4), opposite())
        GUIDE: text.title(label("%(title)s"))
        SCALE: cat(dim(1))
        SCALE: linear(dim(2))
        SCALE: cat(dim(4))
        ELEMENT: interval(position(summary.%(function)s(%(xvar)s*%(yvar)s*1*%(rowvar)s)), shape.interior(shape.square))
        PAGE: end()
        END GPL.   """ % locals()
    try:
        spss.Submit(cmd)
    except:
        print '\n***Following syntax generated an error',cmd
        raise

def findNbDistinctVal(varname):
    """ Finds the number of distinct values of variable varname """

    dsname = spssaux.GetActiveDatasetName()
    if dsname is None:
        dsname = "T" + str(random.random())
        spss.Submit("DATASET NAME %s." % dsname)
    dsnTemp = "T" + str(random.random())
    spss.Submit(r"""
        DATASET DECLARE %(dsnTemp)s.
        AGG OUTFILE=%(dsnTemp)s
          /BREAK=%(varname)s
          /n=n.
        DATASET ACTIVATE %(dsnTemp)s. """ % locals())
    nbCases = spss.GetCaseCount()
    spss.Submit(r"""
        DATASET ACTIVATE %(dsname)s.
        DATASET CLOSE %(dsnTemp)s.  """ % locals())
    return nbCases

if __name__ == '__main__':

    dsn1 = "T" + str(random.random())
    spss.Submit(r"""
        SET MPRINT=YES /PRINTBACK=YES.
        GET FILE="C:\Program Files\SPSS\employee data.sav".
        DATASET NAME %(dsn1)s. """ % locals() )
    doGraphSize(title='Graph with 3 panels', xvar='gender', yvar='salary', rowvar='jobcat', function='mean')
    doGraph(title='Graph with 10 panels', xvar='gender', yvar='salary', rowvar='educ', function='mean')
